/*
 * Copyright (C) 2017  Kevin Whitaker <eyecreate@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef GROOVEPLAYER_H
#define GROOVEPLAYER_H

#include <Wt/Dbo/Session>
#include <Wt/Dbo/backend/Sqlite3>
#include <Wt/Dbo/FixedSqlConnectionPool>
#include <list>
#include <thread>
#include <chrono>
#include "db/User.h"
#include "db/AudioTrack.h"
#include <groove/groove.h>
#include <experimental/filesystem> //TODO:Change to non-gcc way when officially using c++17
namespace std {
    namespace filesystem =  experimental::filesystem;
}

class GroovePlayerMgr
{
public:
    std::filesystem::path musicScanDir;
    static GroovePlayerMgr* getInstance() {
        static GroovePlayerMgr instance("music.db");
        return &instance;
    };
    GroovePlayerMgr ( GroovePlayerMgr const&) = delete;
    void operator=( GroovePlayerMgr const&) = delete;
    void shutdown();
    
    struct GroovePlaylist* currentPlaylist;
    struct GroovePlayer* currentPlayer;
    AudioTrack currentTrack;
    std::list<AudioTrack> currentVoteBatch;
    std::list<std::pair<AudioTrack, int>> currentVoteStatus;
    std::list<User> currentUsersVoted;
    bool voteEndedButNotNextTrackYet = false;
    std::list<User> currentSkipRequester;
    std::chrono::steady_clock::time_point skipRequestedAt;
    
    enum ScanResults {ACCEPTED = true, MISSING_ARTIST_TAG = false, MISSING_ALBUM_TAG = false, MISSING_TITLE_TAG = false, MISSING_GENRE_TAG = false, DUPLICATE_TRACK = false, MISSING_COVERART_TAG = false, MISSING_MULTIPLE_TAGS = false};
    ScanResults addFileToTrackDBIfTagged(Wt::Dbo::Session* session, std::filesystem::path file);
    
    enum PlayerEventType {NOTHING, GROOVE_NOWPLAYING, VOTING_ENDED, VOTE_CAST, PLAYING_PAUSED, PLAYING_RESUMED, SKIP_REQUESTED, SKIP_VOTE_CAST_AGAINST, SKIP_VOTING_ENDED, ADMIN_FORCE_SKIP};
    struct PlayerEvent {
        PlayerEventType eventType;
        User userInvolved;
        std::list<AudioTrack> tracksInvolved;
    };
    std::list<AudioTrack> requestQueue;
    std::list<PlayerEvent> lastInternalEvents;
    Wt::Dbo::backend::Sqlite3* sqliteConnection;
    Wt::Dbo::FixedSqlConnectionPool* connectionPool;
    
    void getPictureFromTrack(AudioTrack* trackToFill);
private:
    GroovePlayerMgr (std::string dbFile);
    ~GroovePlayerMgr();
    bool continueEventLoop = true;
    
    std::thread* grooveEvents;
    void grooveEventLoop();
    std::list<AudioTrack> getNextVoteBatch(Wt::Dbo::Session* session);
    std::thread* grooveAudioScanner;
    void grooveAudioScannerLoop();
    PlayerEvent getNextPlayerEvent(Wt::Dbo::Session* session);
    void removeOrphanedTracks(Wt::Dbo::Session* session);
    
    AudioTrack getCurrentTrackDB(Wt::Dbo::Session* session);
    
};

#endif // GROOVEPLAYER_H
